import * as R from 'ramda';
import { createPatch } from 'rfc6902';
import { OBJ } from 'ploson';

export const idUniquesInFirst = R.differenceWith(
  ({ id: id1 } = OBJ, { id: id2 } = OBJ) => id1 === id2,
);

export const configuredCollectionDiff =
  ({ idKey = 'id', ignoreKeys, skipIf, transformAdded, transformUpdated } = {}) =>
  (prev) =>
    R.converge(R.mergeLeft, [
      R.reduce(
        (acc, item) => {
          if (skipIf && skipIf(item)) return acc;
          const { [idKey]: id } = item;
          const prevItem = R.find(R.propEq('id', id))(prev);
          if (prevItem) {
            const args = R.when(() => ignoreKeys, R.map(R.omit(ignoreKeys)))([prevItem, item]);
            const patch = createPatch(...args);
            return R.isEmpty(patch)
              ? acc
              : {
                  ...acc,
                  updated: [
                    ...acc.updated,
                    transformUpdated ? transformUpdated(item, prevItem, patch) : item,
                  ],
                };
          }
          return { ...acc, added: [...acc.added, transformAdded ? transformAdded(item) : item] };
        },
        { added: [], updated: [] },
      ),
      R.pipe(idUniquesInFirst(prev), R.objOf('removed')),
    ]);

export const collectionDiff = configuredCollectionDiff();
